@@ -23,18 +23,19 @@ import("core.base.option")
 import("core.project.config")
 import("core.tool.linker")
 import("core.tool.compiler")
+import("core.tool.toolchain")
 import("core.cache.memcache")
 import("lib.detect.find_tool")
 
 -- translate paths
-function _translate_paths(package, paths)
-    if paths and is_host("windows") and (package:is_plat("mingw") or package:is_plat("msys") or package:is_plat("cygwin")) then
+function _translate_paths(paths)
+    if paths and is_host("windows") then
         if type(paths) == "string" then
-            return (paths:gsub("\\", "/"))
+            return path.unix(paths)
         elseif type(paths) == "table" then
             local result = {}
             for _, p in ipairs(paths) do
-                table.insert(result, (p:gsub("\\", "/")))
+                table.insert(result, path.unix(p))
             end
             return result
         end
@@ -42,15 +43,41 @@ function _translate_paths(package, paths)
     return paths
 end
 
+-- translate cygwin paths
+function _translate_cygwin_paths(paths)
+    if type(paths) == "string" then
+        return path.cygwin(paths)
+    elseif type(paths) == "table" then
+        local result = {}
+        for _, p in ipairs(paths) do
+            table.insert(result, path.cygwin(p))
+        end
+        return result
+    end
+    return paths
+end
+
 -- translate windows bin path
 function _translate_windows_bin_path(bin_path)
     if bin_path then
         local argv = os.argv(bin_path)
-        argv[1] = argv[1]:gsub("\\", "/") .. ".exe"
+        argv[1] = path.unix(argv[1]) .. ".exe"
         return os.args(argv)
     end
 end
 
+-- get msvc
+function _get_msvc(package)
+    local msvc = package:toolchain("msvc")
+    assert(msvc:check(), "vs not found!") -- we need to check vs envs if it has been not checked yet
+    return msvc
+end
+
+-- get msvc run environments
+function _get_msvc_runenvs(package)
+    return os.joinenvs(_get_msvc(package):runenvs())
+end
+
 -- map compiler flags
 function _map_compflags(package, langkind, name, values)
     return compiler.map_flags(langkind, name, values, {target = package})
@@ -96,7 +123,7 @@ function _get_configs(package, configs)
 
     -- add prefix
     local configs = configs or {}
-    table.insert(configs, "--prefix=" .. _translate_paths(package, package:installdir()))
+    table.insert(configs, "--prefix=" .. _translate_paths(package:installdir()))
 
     -- add host for cross-complation
     if not configs.host and _is_cross_compilation(package) then
@@ -161,8 +188,8 @@ function _get_cflags_from_packagedeps(package, opt)
             local fetchinfo = dep:fetch({external = false})
             if fetchinfo then
                 table.join2(result, _map_compflags(package, "cxx", "define", fetchinfo.defines))
-                table.join2(result, _translate_paths(package, _map_compflags(package, "cxx", "includedir", fetchinfo.includedirs)))
-                table.join2(result, _translate_paths(package, _map_compflags(package, "cxx", "sysincludedir", fetchinfo.sysincludedirs)))
+                table.join2(result, _translate_paths(_map_compflags(package, "cxx", "includedir", fetchinfo.includedirs)))
+                table.join2(result, _translate_paths(_map_compflags(package, "cxx", "sysincludedir", fetchinfo.sysincludedirs)))
             end
         end
     end
@@ -177,9 +204,9 @@ function _get_ldflags_from_packagedeps(package, opt)
         if dep then
             local fetchinfo = dep:fetch({external = false})
             if fetchinfo then
-                table.join2(result, _translate_paths(package, _map_linkflags(package, "binary", {"cxx"}, "linkdir", fetchinfo.linkdirs)))
+                table.join2(result, _translate_paths(_map_linkflags(package, "binary", {"cxx"}, "linkdir", fetchinfo.linkdirs)))
                 table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "link", fetchinfo.links))
-                table.join2(result, _translate_paths(package, _map_linkflags(package, "binary", {"cxx"}, "syslink", fetchinfo.syslinks)))
+                table.join2(result, _translate_paths(_map_linkflags(package, "binary", {"cxx"}, "syslink", fetchinfo.syslinks)))
                 table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "framework", fetchinfo.frameworks))
             end
         end
@@ -199,11 +226,13 @@ function buildenvs(package, opt)
         cxxflags = table.join(table.wrap(package:config("cxflags")), package:config("cxxflags"))
         asflags  = table.copy(table.wrap(package:config("asflags")))
         ldflags  = table.copy(table.wrap(package:config("ldflags")))
+        shflags  = table.copy(table.wrap(package:config("shflags")))
         if package:is_plat("linux") and package:is_arch("i386") then
             table.insert(cflags,   "-m32")
             table.insert(cxxflags, "-m32")
             table.insert(asflags,  "-m32")
             table.insert(ldflags,  "-m32")
+            table.insert(shflags,  "-m32")
         end
         table.join2(cflags,   opt.cflags)
         table.join2(cflags,   opt.cxflags)
@@ -212,15 +241,12 @@ function buildenvs(package, opt)
         table.join2(cppflags, opt.cppflags) -- @see https://github.com/xmake-io/xmake/issues/1688
         table.join2(asflags,  opt.asflags)
         table.join2(ldflags,  opt.ldflags)
+        table.join2(shflags,  opt.shflags)
         table.join2(cflags,   _get_cflags_from_packagedeps(package, opt))
         table.join2(cxxflags, _get_cflags_from_packagedeps(package, opt))
         table.join2(cppflags, _get_cflags_from_packagedeps(package, opt))
         table.join2(ldflags,  _get_ldflags_from_packagedeps(package, opt))
-        envs.CFLAGS    = table.concat(cflags, ' ')
-        envs.CXXFLAGS  = table.concat(cxxflags, ' ')
-        envs.CPPFLAGS  = table.concat(cppflags, ' ')
-        envs.ASFLAGS   = table.concat(asflags, ' ')
-        envs.LDFLAGS   = table.concat(ldflags, ' ')
+        table.join2(shflags,  _get_ldflags_from_packagedeps(package, opt))
     else
         cross = true
         cppflags = {}
@@ -282,23 +308,40 @@ function buildenvs(package, opt)
         table.join2(cxxflags, package:_generate_lto_configs("cxx").cxxflags)
         table.join2(ldflags, package:_generate_lto_configs().ldflags)
     end
+    local runtimes = package:runtimes()
+    if runtimes then
+        local fake_target = {is_shared = function(_) return false end,
+                             sourcekinds = function(_) return "cxx" end}
+        table.join2(cxxflags, _map_compflags(fake_target, "cxx", "runtime", runtimes))
+        table.join2(ldflags, _map_linkflags(fake_target, "binary", {"cxx"}, "runtime", runtimes))
+        fake_target = {is_shared = function(_) return true end,
+                       sourcekinds = function(_) return "cxx" end}
+        table.join2(shflags, _map_linkflags(fake_target, "shared", {"cxx"}, "runtime", runtimes))
+    end
     if package:config("asan") then
         table.join2(cflags, package:_generate_sanitizer_configs("address", "cc").cflags)
         table.join2(cxxflags, package:_generate_sanitizer_configs("address", "cxx").cxxflags)
         table.join2(ldflags, package:_generate_sanitizer_configs("address").ldflags)
+        table.join2(shflags, package:_generate_sanitizer_configs("address").shflags)
+    end
+    if cflags then
+        envs.CFLAGS    = table.concat(_translate_paths(cflags), ' ')
+    end
+    if cxxflags then
+        envs.CXXFLAGS  = table.concat(_translate_paths(cxxflags), ' ')
+    end
+    if cppflags then
+        envs.CPPFLAGS  = table.concat(_translate_paths(cppflags), ' ')
+    end
+    if asflags then
+        envs.ASFLAGS   = table.concat(_translate_paths(asflags), ' ')
     end
-    envs.CFLAGS    = table.concat(cflags, ' ')
-    envs.CXXFLAGS  = table.concat(cxxflags, ' ')
-    envs.CPPFLAGS  = table.concat(cppflags, ' ')
-    envs.ASFLAGS   = table.concat(asflags, ' ')
     if arflags then
-        envs.ARFLAGS   = table.concat(arflags, ' ')
+        envs.ARFLAGS   = table.concat(_translate_paths(arflags), ' ')
     end
-    if ldflags then
-        envs.LDFLAGS   = table.concat(ldflags, ' ')
-    end
-    if shflags then
-        envs.SHFLAGS   = table.concat(shflags, ' ')
+    if ldflags or shflags then
+        -- autoconf does not use SHFLAGS
+        envs.LDFLAGS   = table.concat(_translate_paths(table.join(ldflags or {}, shflags)), ' ')
     end
 
     -- cross-compilation? pass the full build environments
@@ -361,6 +404,9 @@ function buildenvs(package, opt)
             name = name:gsub("gcc%-", "g++-")
             envs.CXX = dir and path.join(dir, name) or name
         end
+    elseif package:is_plat("windows") and not package:config("toolchains") then
+        envs.PATH = os.getenv("PATH") -- we need to reserve PATH on msys2
+        envs = os.joinenvs(envs, _get_msvc(package):runenvs())
     end
     if is_host("windows") then
         envs.CC       = _translate_windows_bin_path(envs.CC)
@@ -390,8 +436,16 @@ function buildenvs(package, opt)
             table.insert(ACLOCAL_PATH, aclocal)
         end
     end
-    envs.ACLOCAL_PATH    = path.joinenv(ACLOCAL_PATH)
-    envs.PKG_CONFIG_PATH = path.joinenv(PKG_CONFIG_PATH)
+    envs.ACLOCAL_PATH = path.joinenv(ACLOCAL_PATH)
+    -- fix PKG_CONFIG_PATH for windows/msys2
+    -- @see https://github.com/xmake-io/xmake-repo/issues/3442
+    if package:is_plat("windows") then
+        -- pkg-config can only support for unix path and env seperator on msys/cygwin
+        PKG_CONFIG_PATH = _translate_cygwin_paths(PKG_CONFIG_PATH)
+        envs.PKG_CONFIG_PATH = path.joinenv(PKG_CONFIG_PATH, ":")
+    else
+        envs.PKG_CONFIG_PATH = path.joinenv(PKG_CONFIG_PATH)
+    end
     return envs
 end
 
@@ -474,7 +528,13 @@ function make(package, argv, opt)
         end
     end
     assert(program, "make not found!")
-    os.vrunv(program, argv)
+
+    if package:is_plat("windows") then
+        local envs = opt.envs or buildenvs(package, opt)
+        os.vrunv(program, argv, {envs = envs})
+    else
+        os.vrunv(program, argv)
+    end
 end
 
 -- build package
@@ -531,4 +591,3 @@ function install(package, configs, opt)
     end
     make(package, argv, opt)
 end
-
